﻿/*
 * Copyright (C) by Peter Herzog 
 * 
 * WriteMagicBytesToUSB
 * Wird verwendet um auf einem USB-Stick die Kennung zu ändern.
 * Sie muss    0x9d 0x2a 0x44 0x7b   lauten damit ein Wetab den USB-Stick booten kann
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Management;
using System.Diagnostics;
using System.Runtime.InteropServices;
using Microsoft.Win32.SafeHandles;


namespace writemagicbytestousb
{

    public partial class Form1 : Form
    {
        /// <summary>
        /// Die CreateFile Deklaration für den Systemzugriff
        /// </summary>
        /// <param name="fileName"></param>
        /// <param name="fileAccess"></param>
        /// <param name="fileShare"></param>
        /// <param name="securityAttributes"></param>
        /// <param name="creationDisposition"></param>
        /// <param name="flags"></param>
        /// <param name="template"></param>
        /// <returns></returns>
        [DllImport("Kernel32.dll", SetLastError = true, CharSet = CharSet.Auto)]
        static extern SafeFileHandle CreateFile(string fileName,
            [MarshalAs(UnmanagedType.U4)] FileAccess fileAccess,
            [MarshalAs(UnmanagedType.U4)] FileShare fileShare,
            int securityAttributes,
            [MarshalAs(UnmanagedType.U4)] FileMode creationDisposition,
            int flags,
            IntPtr template);

        const int FILE_FLAG_NO_BUFFERING = 0x20000000;
        public Form1()
        {
            InitializeComponent();
            // Die Listbox mit den USB-Stick Daten füllen
            filllistbox();
        }

        private void Reload_Click(object sender, EventArgs e)
        {
            // wird der Button geklickt, nochmal füllen
            filllistbox();
        }
        /// <summary>
        /// Das füllt die Listbox mit den Daten der gefundenen USB-Sticks
        /// </summary>
        private void filllistbox()
        {
            // Es wird mit try catch eine grobe Fehlermeldung verhindert und wenn, dann ein Eintrag im Protokoll gemacht
            try
            {
                string lcs;
                string path;
                // Die Liste leer machen
                USBListe.Items.Clear();

                // Definiere mal die Liste der Laufwerke
                DriveInfo[] colDrives = DriveInfo.GetDrives();
                
                // Geh mal alle durch die gefunden wurden
                foreach (DriveInfo di in colDrives)
                {
                    // Ganz speziell nur die Laufwerke, welche als Removable gekennzeichnet sind.
                    // Es gibt noch zusätzlich folgende Typen
                    // System.IO.DriveType.CDRom 
                    // System.IO.DriveType.Fixed
                    // System.IO.DriveType.Network
                    // System.IO.DriveType.NoRootDirectory
                    // System.IO.DriveType.Ram
                    // System.IO.DriveType.Unknown

                    // hole den echten Pfad
                    path = getrealpath(di.Name.Substring(0, 1));

                    if (sec1.Checked)
                    {
                        // Die security Version
                        if (di.DriveType == System.IO.DriveType.Removable)
                        {
                            // Ich stelle mir einen String zusammen.
                            // an der ersten Stelle muss zwingen der Laufwerksbuchstabe sein
                            lcs = di.Name + " " + di.VolumeLabel.ToString();
                            lcs = lcs + " " + path + " - ";
                            lcs = lcs + ",";
                            lcs = lcs + di.DriveFormat.ToString();
                            lcs = lcs + ",";
                            lcs = lcs + di.AvailableFreeSpace.ToString();
                            // Hinzufügen der Zeile
                            USBListe.Items.Add(lcs);
                            // Ist die Liste leer, wird diese erste Zeile markiert
                            if (USBListe.SelectedIndex == -1)
                                USBListe.SelectedItem = lcs;
                        }
                    }
                    if (sec2.Checked)
                    {
                        // hier werden alle Laufwerke gezeigt
                        // Ich stelle mir einen String zusammen.
                        // an der ersten Stelle muss zwingen der Laufwerksbuchstabe sein

                        
                        lcs = di.Name + " ";
                        lcs = lcs + path + " - ";
                        try
                        {
                            lcs = lcs + di.VolumeLabel.ToString();
                        }
                        catch
                        {
                            lcs = lcs + "unknown";
                        }
                        lcs = lcs + ",";
                        try
                        {
                            lcs = lcs + di.DriveType;                        
                        }
                        catch
                        {
                            lcs = lcs + " Type: unknown";
                        }
                        lcs = lcs + ",";
                        try
                        {
                            lcs = lcs + di.DriveFormat.ToString();
                        }
                        catch 
                        {
                            lcs = lcs + "Format: unknown";
                        }
                        lcs = lcs + ",";
                        try
                        {
                            lcs = lcs + di.AvailableFreeSpace.ToString();
                        }
                        catch
                        {
                            lcs = lcs + "Size: unknown" ;
                        }
                            // Hinzufügen der Zeile
                        USBListe.Items.Add(lcs);
                        // Ist die Liste leer, wird diese erste Zeile markiert
                        if (USBListe.SelectedIndex == -1 && di.DriveType == System.IO.DriveType.Removable)
                            USBListe.SelectedItem = lcs;
                    }


                }
            }
            catch (Exception ex)
            {
                // Es ist was passiert, also Fehler ins Eventlog schreiben
                string SourceName = "WindowsService.ExceptionLog";
                if (!EventLog.SourceExists(SourceName))
                {
                    EventLog.CreateEventSource(SourceName, "Application");
                }

                EventLog eventLog = new EventLog();
                eventLog.Source = SourceName;
                string message = string.Format("Exception: {0} \n\nStack: {1}", ex.Message, ex.StackTrace);
                eventLog.WriteEntry(message, EventLogEntryType.Error);

            }
        }
        /// <summary>
        /// Liest die erste 512 Bytes vom Stick 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void readsector_Click(object sender, EventArgs e)
        {
            // Es wird mit try catch eine grobe Fehlermeldung verhindert und wenn, dann ein Eintrag im Protokoll gemacht
            try
            {
                byte[] lcs;
                // lcs soll unser 512Byte-Topf werden
                lcs = new byte[512];

                // So sieht der String im besten Fall dann aus
                // wird in den nächsten Zeilen überschrieben
                string path = "";
                // Hole die den ersten Buchstaben aus der selektierten Zeile
                string driveletter = USBListe.SelectedItem.ToString().Substring(0, 1);
                // hole den echten Pfad
                path = getrealpath(driveletter);
                // Wir erzeugen eine Art Filehandle, auch wenn es sich dabei um ein Laufwerk handelt.
                // FILE_FLAG_NO_BUFFERING ist sehr wichtig, denn sonst wird gebuffert und ein 
                // rausziehen des Sticks nach einem Schreiben erzeugt defekte Sticks oder es ist nicht geschrieben
                SafeFileHandle handle = CreateFile(path, FileAccess.Read, FileShare.ReadWrite, 0, FileMode.Open, FILE_FLAG_NO_BUFFERING, IntPtr.Zero);
                // Wenn das handle OK ist
                if (handle.IsInvalid)
                {
                    //Autsch. 
                    MessageBox.Show(this, "Laufwerk nicht ansprechbar", "Zugriffsfehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                // Filestream als die 512 Byte reinholen
                FileStream fs = new FileStream(handle, FileAccess.Read);
                fs.Read(lcs, 0, 512);
                fs.Close();

                // Fülle die Felder in der Maske 
                // ToString("x") erzeugt einfaches Hex
                // ToString("X") erzeugt geuppertes Hex
                // ToString("X2") erzeugt immer 2stelliges geuppertes Hex
                Byte1.Text = lcs[440].ToString("X2");
                Byte2.Text = lcs[441].ToString("X2");
                Byte3.Text = lcs[442].ToString("X2");
                Byte4.Text = lcs[443].ToString("X2");

                // Handle zumachen 
                handle.Close();
                // Schreiben erlauben
                writesector.Enabled = true;
            }
            catch (Exception ex)
            {
                // Es ist was passiert, also Fehler ins Eventlog schreiben
                string SourceName = "WindowsService.ExceptionLog";
                if (!EventLog.SourceExists(SourceName))
                {
                    EventLog.CreateEventSource(SourceName, "Application");
                }

                EventLog eventLog = new EventLog();
                eventLog.Source = SourceName;
                string message = string.Format("Exception: {0} \n\nStack: {1}", ex.Message, ex.StackTrace);
                eventLog.WriteEntry(message, EventLogEntryType.Error);

            }
        }
        /// <summary>
        /// schreibt die vorher gelesenen 512 Bytes wieder raus
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void writesector_Click(object sender, EventArgs e)
        {
            // Es wird mit try catch eine grobe Fehlermeldung verhindert und wenn, dann ein Eintrag im Protokoll gemacht
            try
            {
                byte[] lcs;
                // lcs soll unser 512Byte-Topf werden
                lcs = new byte[512];

                // So sieht der String im besten Fall dann aus
                // wird in den nächsten Zeilen überschrieben
                string path = "";
                // Hole die den ersten Buchstaben aus der selektierten Zeile
                string driveletter = USBListe.SelectedItem.ToString().Substring(0, 1);
                // hole den echten Pfad
                path = getrealpath(driveletter);
                // Wir erzeugen eine Art Filehandle, auch wenn es sich dabei um ein Laufwerk handelt.
                // FILE_FLAG_NO_BUFFERING ist sehr wichtig, denn sonst wird gebuffert und ein 
                // rausziehen des Sticks nach einem Schreiben erzeugt defekte Sticks oder es ist nicht geschrieben
                SafeFileHandle handle = CreateFile(path, FileAccess.Read, FileShare.ReadWrite, 0, FileMode.Open, FILE_FLAG_NO_BUFFERING, IntPtr.Zero);
                // Wenn das handle OK ist
                if (handle.IsInvalid)
                {
                    //Schon wieder Autsch.
                    MessageBox.Show(this, "Laufwerk nicht ansprechbar", "Zugriffsfehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                // Filestream als die 512 Byte reinholen
                FileStream fs = new FileStream(handle, FileAccess.Read);
                fs.Read(lcs, 0, 512);
                fs.Close();
                // Kann gleich wieder zu gemacht werden
                handle.Close();


                // Hole die 4 Bytes aus den Feldern am Bildschirm
                // byte.Parse ist ein geiler Befehl :-)
                lcs[440] = byte.Parse(Byte1.Text, System.Globalization.NumberStyles.HexNumber);
                lcs[441] = byte.Parse(Byte2.Text, System.Globalization.NumberStyles.HexNumber);
                lcs[442] = byte.Parse(Byte3.Text, System.Globalization.NumberStyles.HexNumber);
                lcs[443] = byte.Parse(Byte4.Text, System.Globalization.NumberStyles.HexNumber);

                // Das sind die berühmten
                //lcs[440] = 0x9d;
                //lcs[441] = 0x2a;
                //lcs[442] = 0x44;
                //lcs[443] = 0x7b;

                // Abfrage
                if (MessageBox.Show(this, "Do you want to write the Data to \n" + USBListe.SelectedItem.ToString() + "?", "Access drive physicaly", MessageBoxButtons.YesNo, MessageBoxIcon.Hand) == DialogResult.Yes)
                {
                    // extra nochmal in ein Try Catch rein  
                    try
                    {
                        // Diesmal mit FileAccess.Write aufmachen
                        SafeFileHandle handle2 = CreateFile(path, FileAccess.Write, FileShare.ReadWrite, 0, FileMode.Open, FILE_FLAG_NO_BUFFERING, IntPtr.Zero);
                        // Ist es OK
                        if (handle2.IsInvalid)
                        {
                            // Tja. Was könnte es gewesen sein. Rausgezogen vor dem Schreiben... Irgendwas
                            MessageBox.Show(this, "Laufwerk zum Schreiben nicht ansprechbar", "Zugriffsfehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
                            return;
                        }

                        // Den Filestream den wir vorher reingelesen haben und wo die 4 bytes reingeschrieben wurden
                        // wird nun wieder rausgepfeffert
                        FileStream fs2 = new FileStream(handle2, FileAccess.Write);
                        fs2.Write(lcs, 0, 512);
                        fs2.Close();
                        handle2.Close();
                    }
                    catch (IOException oerr)
                    {
                        // Ui irgendwas blödes ist passiert.
                        MessageBox.Show(this, oerr.Message, "Zugriffsfehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }

                writesector.Enabled = false;
            }
            catch (Exception ex)
            {
                // Es ist was passiert, also Fehler ins Eventlog schreiben
                string SourceName = "WindowsService.ExceptionLog";
                if (!EventLog.SourceExists(SourceName))
                {
                    EventLog.CreateEventSource(SourceName, "Application");
                }

                EventLog eventLog = new EventLog();
                eventLog.Source = SourceName;
                string message = string.Format("Exception: {0} \n\nStack: {1}", ex.Message, ex.StackTrace);
                eventLog.WriteEntry(message, EventLogEntryType.Error);

            }
        }
        /// <summary>
        /// Hole den echten Pfad um auf das Laufwerk zuzugreifen
        /// </summary>
        /// <param name="tcdriveletter"></param>
        /// <returns></returns>
        private string getrealpath(string tcdriveletter)
        {
            string path = "not physikal";
            // Wir suchen mal alle Partitions die zum Driveletter passen
            using (ManagementObject mo = new ManagementObject(@"Win32_LogicalDisk='" + tcdriveletter + ":'"))
            {
                // und nun durch die Partition zum richtigen 
                foreach (ManagementObject b in mo.GetRelated("Win32_DiskPartition"))
                {
                    // physikalischen Laufwerk
                    foreach (ManagementBaseObject c in b.GetRelated("Win32_Diskdrive"))
                        // und dessen Pfad nach path schreiben
                        path = c["Name"].ToString();
                }
            }
            return path;
        }
        /// <summary>
        /// Setze die berühmten 4 Bytes in die Maske
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void setbutton_Click(object sender, EventArgs e)
        {
            Byte1.Text = "9d";
            Byte2.Text = "2a";
            Byte3.Text = "44";
            Byte4.Text = "7b";
        }
        /// <summary>
        /// verhindere Nicht-Hex-Zahlen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Byte1_KeyPress(object sender, KeyPressEventArgs e)
        {
            byte[] b;
            string s = "";
            s += e.KeyChar;
            b = Encoding.ASCII.GetBytes(s);
            if (e.KeyChar != (char)Keys.Back)
            {
                //check if character is any 0-9/a-f/A-F
                if (!(((0x30 <= b[0]) && (b[0] <= 0x39)) || ((0x41 <= b[0]) && (b[0] <= 0x46)) || ((0x61 <= b[0]) && (b[0] <= 0x66))))
                {
                    e.Handled = true;
                    e.KeyChar = ' ';
                }
            }
        }
        /// <summary>
        /// verhindere Nicht-Hex-Zahlen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Byte2_KeyPress(object sender, KeyPressEventArgs e)
        {
            byte[] b;
            string s = "";
            s += e.KeyChar;
            b = Encoding.ASCII.GetBytes(s);
            if (e.KeyChar != (char)Keys.Back)
            {
                //check if character is any 0-9/a-f/A-F
                if (!(((0x30 <= b[0]) && (b[0] <= 0x39)) || ((0x41 <= b[0]) && (b[0] <= 0x46)) || ((0x61 <= b[0]) && (b[0] <= 0x66))))
                {
                    e.Handled = true;
                    e.KeyChar = ' ';
                }
            }
        }
        /// <summary>
        /// verhindere Nicht-Hex-Zahlen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Byte3_KeyPress(object sender, KeyPressEventArgs e)
        {
            byte[] b;
            string s = "";
            s += e.KeyChar;
            b = Encoding.ASCII.GetBytes(s);
            if (e.KeyChar != (char)Keys.Back)
            {
                //check if character is any 0-9/a-f/A-F
                if (!(((0x30 <= b[0]) && (b[0] <= 0x39)) || ((0x41 <= b[0]) && (b[0] <= 0x46)) || ((0x61 <= b[0]) && (b[0] <= 0x66))))
                {
                    e.Handled = true;
                    e.KeyChar = ' ';
                }
            }
        }
        /// <summary>
        /// verhindere Nicht-Hex-Zahlen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Byte4_KeyPress(object sender, KeyPressEventArgs e)
        {
            byte[] b;
            string s = "";
            s += e.KeyChar;
            b = Encoding.ASCII.GetBytes(s);
            if (e.KeyChar != (char)Keys.Back)
            {
                //check if character is any 0-9/a-f/A-F
                if (!(((0x30 <= b[0]) && (b[0] <= 0x39)) || ((0x41 <= b[0]) && (b[0] <= 0x46)) || ((0x61 <= b[0]) && (b[0] <= 0x66))))
                {
                    e.Handled = true;
                    e.KeyChar = ' ';
                }
            }
        }
        /// <summary>
        /// Wird ein anderer USB-Stick angewählt, so darf nicht mehr geschrieben werden
        /// Also einfach disablen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void USBListe_SelectedValueChanged(object sender, EventArgs e)
        {
            writesector.Enabled = false;
        }
        /// <summary>
        /// Klick auf den Info-Button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void info_Click(object sender, EventArgs e)
        {
            infomessage();
        }
        /// <summary>
        /// Zeig die Copyright nochmal an
        /// </summary>
        private void infomessage()
        {
            string lcs = "";
            lcs = lcs + "WeTab USB Stick Signer\n\n";
            lcs = lcs + "WeTab-Community Version\n\n";
            lcs = lcs + "Achtung: \n\n";
            lcs = lcs + "Ich übernehme keine Garantie und die Verwendung\ndieses Tools geht auf eigene Gefahr\n\n";
            lcs = lcs + "Es wurde nach besten Wissen und Gewissen programmiert,\naber nobody ist Fehlerfrei, also kann es auch mir passieren\n";
            lcs = lcs + "dass sich ein Fehler eingeschlichen hat.\n\n";
            lcs = lcs + "Attention\n";
            lcs = lcs + "I assume no warranty, and the use of this tool is at your own risk\n";
            lcs = lcs + "Nobody is perfekt, so there may be errors.\n";

            lcs = lcs + "(c)2011 by Peter Herzog peter@bgl1.de (Forumsname pmaler)\nSourcecode unter GPL zu finden unter www.pherzog.net";

            MessageBox.Show(this, lcs, "Info", MessageBoxButtons.OK, MessageBoxIcon.Warning);

        }


        class TransparentLabel : Panel
        {
            private String mText;
            [Browsable(true)]
            public override String Text
            {
                get { return mText; }
                set { mText = value; this.Invalidate(); }
            }
            protected new void Invalidate()
            {
                if (this.Parent != null) this.Parent.Invalidate(this.Bounds);
                base.Invalidate();
            }
            protected override CreateParams CreateParams
            {
                get
                {
                    CreateParams cp = base.CreateParams;
                    cp.ExStyle |= 0x20;  // Turn on WS_EX_TRANSPARENT
                    return cp;
                }
            }
            protected override void OnPaintBackground(PaintEventArgs e)
            {
                // Prevent erasing background
            }
            protected override void OnPaint(PaintEventArgs e)
            {
                // Paint text
                Rectangle rc = new Rectangle(0, 0, this.Width, this.Height);
                TextRenderer.DrawText(e.Graphics, this.Text, this.Font, rc, this.ForeColor, Color.Transparent);
            }
        }
   
    }
}
